@@ -36,8 +36,6 @@
  */
 #define IMAGE_VIEW_ZOOM_MULTIPLIER 1.05
 
-#define NO_DEBUG
-
 /* States for automatically adjusting the zoom factor */
 typedef enum {
 	ZOOM_MODE_FIT,		/* Image is fitted to scroll view even if the latter changes size */
@@ -72,6 +70,7 @@ struct _EogScrollViewPrivate {
 	GtkAdjustment *vadj;
 	GtkWidget *hbar;
 	GtkWidget *vbar;
+	GtkWidget *menu;
 
 	/* actual image */
 	EogImage *image;
@@ -131,6 +130,9 @@ static void set_zoom_fit (EogScrollView *view);
 static void request_paint_area (EogScrollView *view, GdkRectangle *area);
 static void set_minimum_zoom_factor (EogScrollView *view);
 
+#define EOG_SCROLL_VIEW_GET_PRIVATE(object) \
+	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EOG_TYPE_SCROLL_VIEW, EogScrollViewPrivate))
+
 G_DEFINE_TYPE (EogScrollView, eog_scroll_view, GTK_TYPE_TABLE)
 
 
@@ -511,6 +513,7 @@ paint_rectangle (EogScrollView *view, ArtIRect *rect, GdkInterpType interp_type)
 	ArtIRect r, d;
 	GdkPixbuf *tmp;
 	int check_size;
+	char *str;
 	guint32 check_1 = 0;
 	guint32 check_2 = 0;
 
@@ -607,7 +610,6 @@ paint_rectangle (EogScrollView *view, ArtIRect *rect, GdkInterpType interp_type)
 	if (art_irect_empty (&d))
 		return;
 
-	char *str;
 	switch (interp_type) {
 	case GDK_INTERP_NEAREST:
 		str = "NEAREST";
@@ -1217,7 +1219,8 @@ eog_scroll_view_button_press_event (GtkWidget *widget, GdkEventButton *event, gp
 	switch (event->button) {
 		case 1:
 		case 2:
-                        if (event->button == 1 && !(event->state & GDK_CONTROL_MASK))
+                        if (event->button == 1 && !priv->scroll_wheel_zoom && 
+			    !(event->state & GDK_CONTROL_MASK))
 				break;
 
 			if (is_image_movable (view)) {
@@ -1903,8 +1906,7 @@ eog_scroll_view_init (EogScrollView *view)
 {
 	EogScrollViewPrivate *priv;
 
-	priv = g_new0 (EogScrollViewPrivate, 1);
-	view->priv = priv;
+	priv = view->priv = EOG_SCROLL_VIEW_GET_PRIVATE (view);
 
 	priv->zoom = 1.0;
 	priv->min_zoom = MIN_ZOOM_FACTOR;
@@ -1920,6 +1922,7 @@ eog_scroll_view_init (EogScrollView *view)
 	priv->transp_style = EOG_TRANSP_BACKGROUND;
 	priv->transp_color = 0;
 	priv->cursor = EOG_SCROLL_VIEW_CURSOR_NORMAL;
+	priv->menu = NULL;
 }
 
 static void
@@ -1948,20 +1951,6 @@ eog_scroll_view_dispose (GObject *object)
 	G_OBJECT_CLASS (eog_scroll_view_parent_class)->dispose (object);
 }
 
-static void
-eog_scroll_view_finalize (GObject *object)
-{
-	EogScrollView *view;
-
-	view = EOG_SCROLL_VIEW (object);
-	if (view->priv != 0) {
-		g_free (view->priv);
-		view->priv = 0;
-	}
-
-	G_OBJECT_CLASS (eog_scroll_view_parent_class)->finalize (object);
-}
-
 static void
 eog_scroll_view_class_init (EogScrollViewClass *klass)
 {
@@ -1972,8 +1961,6 @@ eog_scroll_view_class_init (EogScrollViewClass *klass)
 	widget_class = (GtkWidgetClass*) klass;
 
 	gobject_class->dispose = eog_scroll_view_dispose;
-  	gobject_class->finalize = eog_scroll_view_finalize;
-
 
 	view_signals [SIGNAL_ZOOM_CHANGED] =
 		g_signal_new ("zoom_changed",
@@ -1987,6 +1974,8 @@ eog_scroll_view_class_init (EogScrollViewClass *klass)
 
 	widget_class->size_allocate = eog_scroll_view_size_allocate;
 	widget_class->style_set = eog_scroll_view_style_set;
+
+	g_type_class_add_private (klass, sizeof (EogScrollViewPrivate));
 }
 
 
@@ -2065,6 +2054,58 @@ eog_scroll_view_new (void)
 	return widget;
 }
 
+static void 
+eog_scroll_view_popup_menu (EogScrollView *view, GdkEventButton *event)
+{
+	GtkWidget *popup;
+	int button, event_time;
+
+	popup = view->priv->menu;
+	
+	if (event) {
+		button = event->button;
+		event_time = event->time;
+	} else {
+		button = 0;
+		event_time = gtk_get_current_event_time ();
+	}
+	
+	gtk_menu_popup (GTK_MENU (popup), NULL, NULL, NULL, NULL, 
+			button, event_time);
+}
+
+static gboolean
+view_on_button_press_event_cb (GtkWidget *view, GdkEventButton *event, 
+			       gpointer user_data)
+{    
+    /* Ignore double-clicks and triple-clicks */
+    if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
+    {
+	    eog_scroll_view_popup_menu (EOG_SCROLL_VIEW (view), event);
+
+	    return TRUE;
+    }
+    
+    return FALSE;
+}
+
+void
+eog_scroll_view_set_popup (EogScrollView *view,
+			   GtkMenu *menu)
+{
+	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
+	g_return_if_fail (view->priv->menu == NULL);
+
+	view->priv->menu = g_object_ref (menu);
+
+	gtk_menu_attach_to_widget (GTK_MENU (view->priv->menu), 
+				   GTK_WIDGET (view), 
+				   NULL);
+
+	g_signal_connect (G_OBJECT (view), "button_press_event",
+			  G_CALLBACK (view_on_button_press_event_cb), NULL);
+}
+
 void
 eog_scroll_view_set_scroll_wheel_zoom (EogScrollView *view, 
 				       gboolean       scroll_wheel_zoom)
