@@ -57,6 +57,9 @@ static EogStartupFlags flags;
 static gboolean fullscreen = FALSE;
 static gboolean slide_show = FALSE;
 static gboolean disable_collection = FALSE;
+#if HAVE_DBUS
+static gboolean wait = FALSE;
+#endif
 static gchar **startup_files = NULL;
 
 static const GOptionEntry goption_options[] =
@@ -64,12 +67,15 @@ static const GOptionEntry goption_options[] =
 	{ "fullscreen", 'f', 0, G_OPTION_ARG_NONE, &fullscreen, N_("Open in fullscreen mode"), NULL  },
 	{ "disable-image-collection", 'c', 0, G_OPTION_ARG_NONE, &disable_collection, N_("Disable image collection"), NULL  },
 	{ "slide-show", 's', 0, G_OPTION_ARG_NONE, &slide_show, N_("Open in slide show mode"), NULL  },
+#if HAVE_DBUS
+	{ "new-instance", 'n', 0, G_OPTION_ARG_NONE, &wait, N_("Start a new instance instead of reusing an existing one"), NULL },
+#endif
 	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &startup_files, NULL, N_("[FILE...]") },
 	{ NULL }
 };
 
 static void 
-set_startup_flags ()
+set_startup_flags (void)
 {
   if (fullscreen)
     flags |= EOG_STARTUP_FULLSCREEN;
@@ -82,7 +88,7 @@ set_startup_flags ()
 }
 
 static void 
-load_files ()
+load_files (void)
 {
 	GSList *files = NULL;
 
@@ -100,7 +106,7 @@ load_files ()
 
 #ifdef HAVE_DBUS
 static gboolean
-load_files_remote ()
+load_files_remote (void)
 {
 	GError *error = NULL;
 	DBusGConnection *connection;
@@ -190,7 +196,7 @@ main (int argc, char **argv)
 	set_startup_flags ();
 	
 #ifdef HAVE_DBUS
-	if (!eog_application_register_service (EOG_APP)) {
+	if (!wait && !eog_application_register_service (EOG_APP)) {
 		if (load_files_remote ()) {
 			g_object_unref (program);
 			return 0;
@@ -218,8 +224,12 @@ main (int argc, char **argv)
 
 	load_files ();
 
+	gdk_threads_enter ();
+
 	gtk_main ();
 
+	gdk_threads_leave ();
+
 	gnome_accelerators_sync ();
 	
   	if (startup_files)
